module ReportsMash
  class Control
    module InstanceMethods
      attr_writer :env
      attr_reader :env
      def init_plugin(options={})
        # initialize the engine instance
        ReportsMash::Engine.engine = ReportsMash::Engine::Engine.instance
        ReportsMash::Engine.options = options

        $stderr.puts "Initializing plugin ReportsMash - #{Process.pid}\n"
        setup_rack_hooks(options[:config])
        setup_database_hooks(options[:config], ReportsMash::Engine.engine)
        setup_render_hooks(options[:config], ReportsMash::Engine.engine)

        # if server is forking (eg, unicorn), we should not start the worker thread here
        # it will be handled by Engine.transaction_start method
        ReportsMash::Engine.engine.start_worker unless server_is_forking?
      end

      def setup_rack_hooks(config)
          return if config.nil? || !config.respond_to?(:middleware)
          begin
            require 'reportsmash/rack/hooks'
            config.middleware.use ReportsMash::Rack::Hooks
          rescue => e
            #::ReportsMash::Engine.logger.warn("Error installing ReportsMash Rack middleware", e)
          end        
      end

      def setup_database_hooks(config, engine)
        ActiveSupport::Notifications.subscribe('sql.active_record') do |name, start, finish, id, payload|
          unless payload[:name] == 'SCHEMA'
            engine.save_db_transaction(id, (finish-start)*1000.0, payload[:sql], payload[:connection_id])
          end
        end
      end

      def setup_render_hooks(config, engine)
        ActiveSupport::Notifications.subscribe('render_template.action_view') do |name, start, finish, id, payload|
          view_name = payload[:identifier].split('app/views')[1]
          engine.save_render_transaction(id, (finish-start)*1000.0, 'app/views'+view_name)
        end
        ActiveSupport::Notifications.subscribe('render_partial.action_view') do |name, start, finish, id, payload|
          view_name = payload[:identifier].split('app/views')[1]
          Rails.logger.debug "#{view_name} - #{(finish-start)*1000.0}"
          engine.save_render_transaction(id, (finish-start)*1000.0, 'app/views'+view_name)
        end
      end

      def server_is_forking?
        # check for unicorn server
        return true if defined?(::Unicorn) && defined?(::Unicorn::HttpServer) && object_running?(::Unicorn::HttpServer)

        false
      end

      def object_running?(o)
        ObjectSpace.each_object(o) do |exists|
          return true
        end
        return false
      end

    end

    include InstanceMethods
  end
end

