module ReportsMash::Engine
  class Transaction
    attr_accessor :user_params
    attr_accessor :payload
    attr_accessor :db_transactions, :db_duration_ms, :render_transactions, :render_duration_ms
    attr_accessor :external_http_requests, :external_http_duration_ms
    attr_accessor :memcached_ops, :redis_ops

    def initialize(engine, request)
      @engine = engine
      @request = request
      @response = nil
      @tid = SecureRandom.uuid
      @user_params = []
      @start_time = nil
      @total_duration_ms = 0
      @db_duration_ms = 0
      @render_duration_ms = 0
      @payload = {}
      @db_transactions = []
      @render_transactions = []
      @external_http_requests = []
      @external_http_duration_ms = 0
      @memcached_ops = []
      @redis_ops = []
    end

    def tid
      @tid
    end   

    def add_user_params(new_params)
      @user_params.merge!(new_params) if new_params.is_a? Hash
    end
 
    def end(start_time, stop_time, status_code, headers)
      @start_time = start_time
      response_content_type = (headers && headers["Content-Type"]) ? headers["Content-Type"] : ""
      @total_duration_ms = (stop_time - start_time) * 1000.0
      path_params = @request["action_dispatch.request.path_parameters"]
      controller_action = "static_file_server"
      if path_params && path_params.key?(:controller) && path_params.key?(:action)
        controller_action = path_params[:controller]+"#"+path_params[:action]
      end

      @memcached_stats = {}
      mc_duration_ms = 0
      mc_counter = 0
      if @memcached_ops.count > 0
        %w[get set get_multi].each do |op|
          stat = {count: 0, duration_ms: 0, avg_ms: 0}
          @memcached_ops.select{|h| h.key? op.to_sym}.each do |trace|
            stat[:count] += 1
            stat[:duration_ms] += trace[op.to_sym]
            stat[:avg_ms] = (stat[:duration_ms]/Float(stat[:count])).round(2)
            @memcached_stats[op.to_sym] = stat
            mc_duration_ms += stat[:duration_ms]
            mc_counter += 1
          end
        end
        #puts "GOT MC_OPS #{@memcached_ops.inspect}: #{@memcached_stats.inspect}"
      end

      @redis_stats = {}
      redis_duration_ms = 0
      redis_counter = 0
      if @redis_ops.count > 0
        %w[get set].each do |op|
          stat = {count: 0, duration_ms: 0, avg_ms: 0}
          @redis_ops.select{|h| h.key? op.to_sym}.each do |trace|
            stat[:count] += 1
            stat[:duration_ms] += trace[op.to_sym]
            stat[:avg_ms] = (stat[:duration_ms]/Float(stat[:count])).round(2)
            @redis_stats[op.to_sym] = stat
            redis_duration_ms += stat[:duration_ms]
            redis_counter += 1
          end
        end
        #puts "GOT REDIS_OPS #{@redis_ops.inspect}: #{@redis_stats.inspect}"
      end

      @payload = {
        :time_utc => @start_time.utc.to_i,
        :user_ip => @request["REMOTE_ADDR"],
        :user_agent => @request["HTTP_USER_AGENT"],
        :method => @request["REQUEST_METHOD"],
        :http_host => @request["HTTP_HOST"],
        :request_uri => @request["REQUEST_URI"],
        :request_path => @request["REQUEST_PATH"],
        :controller_action => controller_action,
        :response_content_type => response_content_type,
        :status => status_code,
        :total_duration_ms => @total_duration_ms.round(2),
        :db_duration_ms => @db_duration_ms.round(2),
        :render_duration_ms => @render_duration_ms.round(2),
        :external_http_duration_ms => @external_http_duration_ms.round(2),
        :external_memcache_duration_ms => mc_duration_ms.round(2),
        :external_redis_duration_ms => redis_duration_ms.round(2),
        :db_counter => @db_transactions.count,
        :render_counter => @render_transactions.count,
        :external_http_counter => @external_http_requests.count,
        :external_memcache_counter => mc_counter,
        :external_redis_counter => redis_counter,
        :db_transactions => @db_transactions,
        :render_transactions => @render_transactions,
        :external_http_requests => @external_http_requests,
        :external_memcache_requests => @memcached_stats,
        :external_redis_requests => @redis_stats,
        :custom_attributes => @user_params
      }
    end
  end

end

