require 'thread'
require 'typhoeus'
require 'snappy'

module ReportsMash
  module Engine

    # A worker loop executes a set of registered tasks on a single thread.
    # A task is a proc or block with a specified call period in seconds.
    class WorkerLoop

      def initialize
        @engine = ReportsMash::Engine.engine
      end
      def run
        #$stderr.puts "ReportsMash worker running #{@engine}\n"
        authkey = Base64.strict_encode64("#{ENV['REPORTSMASH_APP_ID']}:#{ENV['REPORTSMASH_APP_KEY']}")
        if Rails.env.development?
          remote_host = "https://in.reportsmash.com:8091/w"
        else
          remote_host = "https://in.reportsmash.com:443/w"
        end
        data = {}
        batch = []

        loop do
          sleep 5
          while item = @engine.delivery_queue.pop do
            batch << item.payload
          end
          next if batch.count == 0
          data[:transactions] = batch

          begin 
            json_data = data.to_json
            compressed_data = Snappy.deflate(json_data)
          
            req = Typhoeus::Request.new(remote_host, method: :post, body: compressed_data, headers: { 
              "X-REPORTSMASH-AUTH" => authkey,
              "Content-Length" => compressed_data.size,
              "Content-Type" => "application/octet-stream" })
            req.run
          rescue Exception => e
            $stderr.puts "ReportsMash::Engine.worker - could not send data to servers: #{e.inspect}"
          end
          #data[:transactions].each do |item|
          #  $stderr.puts "Batch running #{item}" if Rails.env.development?
          #end

          data.clear()
          batch.clear()
        end
      end
    end
  end
end

